@@ -205,14 +205,14 @@ warnings.filterwarnings("ignore", ".*ShapelyDeprecationWarning.*")
 # Now, import the libraries you're using
 import shapely
 import cartopy
-def create_table_if_not_exists(cursor):
+def create_table_if_not_exists(cursor, extra_sw):
     """
     Create the 'serie_cardio' table if it does not already exist.
 
     Args:
         cursor (sqlite3.Cursor): Database cursor for executing SQL queries.
     """
-    query = """
+    query = f"""
         CREATE TABLE IF NOT EXISTS moyenne (
             Nrej INTEGER,
             Nacc INTEGER,
@@ -234,10 +234,11 @@ def create_table_if_not_exists(cursor):
             sumStat2 FLOAT,
             n INTEGER,
             flag INTEGER
+            {extra_sw}
         ); """
     cursor.execute(query)
 
-def combine(pathfileout: str, filememory: str) -> None:
+def combine(pathfileout: str, filememory: str, extra_sw) -> None:
     """
     Combine multiple SQLite files into a single output file.
 
@@ -251,17 +252,16 @@ def combine(pathfileout: str, filememory: str) -> None:
     Returns:
         None. The result is saved directly in the specified output file.
     """
-    insert_sql = """
+    insert_sql = f"""
         INSERT INTO moyenne(
             Nrej, Nacc, Nprofile, DATE, lat, lon, boite, id_stn, varno, vcoord, sumx, sumy, sumz, 
-            sumStat, sumx2, sumy2, sumz2, sumStat2, n, flag
+            sumStat, sumx2, sumy2, sumz2, sumStat2, n, flag {extra_sw}
         )
         SELECT 
             Nrej, Nacc, Nprofile, DATE, lat, lon, boite, id_stn, varno, vcoord, sumx, sumy, sumz, 
-            sumStat, sumx2, sumy2, sumz2, sumStat2, n, flag
+            sumStat, sumx2, sumy2, sumz2, sumStat2, n, flag {extra_sw}
         FROM this_avg_db.moyenne
     """
-
     # Use 'with' statement to ensure connection is properly closed
     with sqlite3.connect(pathfileout, uri=True, isolation_level=None, timeout=9999) as conn:
         try:
@@ -270,7 +270,7 @@ def combine(pathfileout: str, filememory: str) -> None:
             conn.execute("PRAGMA synchronous=OFF;")
 
             # Ensure the table exists
-            create_table_if_not_exists(conn)
+            create_table_if_not_exists(conn, extra_sw)
 
             # Attach the source SQLite database
             conn.execute(f"ATTACH DATABASE '{filememory}' AS this_avg_db;")
@@ -285,53 +285,6 @@ def combine(pathfileout: str, filememory: str) -> None:
             print(f"Error combining SQLite files: {e}")
             raise
 
-def remove_combine(pathfileout, filememory):
-
-    """ Creating a single sqlite file from multiple sqlite files
- 
-    args:
-    ----------------
-
-     pathfileout   : output averaging  sqlite file 
-     filememory    : name of the sqlite file in memory to copy
-   
-    output:
-    ----------------
-
-    Nothing
-     A sqlite file is made with all averaged volume scans
-   
-   """
-
-    # write in output averaging  sqlite file
-    conn_pathfileout = sqlite3.connect(pathfileout, uri=True, isolation_level=None, timeout=9999)
-
-    conn_pathfileout.execute("""PRAGMA journal_mode=OFF;""")
-
-    # off the journal
-    create_table_if_not_exists( conn_pathfileout)
-    conn_pathfileout.execute("""PRAGMA journal_mode=OFF;""")
-    # SQLite continues without syncing as soon as it has handed data off to the operating system
-    conn_pathfileout.execute("""PRAGMA synchronous=OFF;""")
-    # Wait to read and write until the next process finishes
-    # attach the sqlite file in memory for one PPI
-    create_table_if_not_exists(conn_pathfileout)
-    conn_pathfileout.execute(f"ATTACH DATABASE '{filememory}' AS this_avg_db;")
-    order_sql = """ INSERT INTO moyenne( Nrej, Nacc, Nprofile, DATE, lat, lon, boite, id_stn, varno , vcoord, sumx, sumy, sumz, sumStat, sumx2,
-                                         sumy2, sumz2, sumStat2, n, flag 
- 
-
-                )
-                    SELECT Nrej, Nacc, Nprofile, DATE, lat, lon, boite, id_stn, varno , vcoord, sumx, sumy, sumz, sumStat, sumx2,
-                                         sumy2, sumz2, sumStat2, n, flag 
- 
- 
-                    FROM  this_avg_db.moyenne"""
-    conn_pathfileout.execute(order_sql) 
-
-    conn_pathfileout.commit()
-    conn_pathfileout.execute(""" DETACH DATABASE this_avg_db """)
-
 def create_and_populate_moyenne_table(
     family,
     new_db_filename: str,
@@ -383,26 +336,6 @@ def create_and_populate_moyenne_table(
         FAM, VCOORD, VCOCRIT, STATB, element, VCOTYP = pikobs.family(family)
         element = ",".join(varnos) if varnos else element
 
-        # Define SQL for GROUP BY and SELECT fields depending on channel/id_stn
-        if channel == 'join' and id_stn == 'all':
-            chan_select = '"join" as Chan,'
-            id_stn_select = 'id_stn as id_stn,'
-            group_by = 'GROUP BY 2, 3, 4, 5, id_stn'
-        elif channel == 'all' and id_stn == 'join':
-            chan_select = f'{VCOORD} as Chan,'
-            id_stn_select = '"join" as id_stn,'
-            group_by = f'GROUP BY 2, 3, 4, 5, {VCOORD}'
-        elif channel == 'all' and id_stn == 'all':
-            chan_select = f'{VCOORD} as Chan,'
-            id_stn_select = 'id_stn as id_stn,'
-            group_by = f'GROUP BY 2, 3, 4, 5, id_stn, {VCOORD}'
-        elif channel == 'join' and id_stn == 'join':
-            chan_select = '"join" as Chan,'
-            id_stn_select = '"join" as id_stn,'
-            group_by = 'GROUP BY 2, 3, 4, 5, date'
-        else:
-            raise ValueError("Unsupported combination of channel and id_stn")
-
         # Region, latitude, longitude and flag criteria
         LAT1, LAT2, LON1, LON2 = pikobs.regions(selected_region)
         
@@ -424,6 +357,32 @@ def create_and_populate_moyenne_table(
         cursor.execute("PRAGMA table_info('DATA')")
         columns = [col[1] for col in cursor.fetchall()]
         has_bias_corr = 'BIAS_CORR' in columns
+          
+        cursor.execute("PRAGMA table_info('HEADER')")
+        columns = [col[1] for col in cursor.fetchall()]
+        WIND_COMP_METHOD = 'WIND_COMP_METHOD' in columns
+        extra_sw = ', WIND_COMP_METHOD' if WIND_COMP_METHOD else ''
+
+        
+        # Define SQL for GROUP BY and SELECT fields depending on channel/id_stn
+        if channel == 'join' and id_stn == 'all':
+            chan_select = '"join" as Chan,'
+            id_stn_select = 'id_stn as id_stn,'
+            group_by = f'GROUP BY 2, 3, 4, 5, id_stn {extra_sw}'
+        elif channel == 'all' and id_stn == 'join':
+            chan_select = f'{VCOORD} as Chan,'
+            id_stn_select = '"join" as id_stn,'
+            group_by = f'GROUP BY 2, 3, 4, 5, {VCOORD} {extra_sw}'
+        elif channel == 'all' and id_stn == 'all':
+            chan_select = f'{VCOORD} as Chan,'
+            id_stn_select = 'id_stn as id_stn,'
+            group_by = f'GROUP BY 2, 3, 4, 5, id_stn, {VCOORD} {extra_sw}'
+        elif channel == 'join' and id_stn == 'join':
+            chan_select = '"join" as Chan,'
+            id_stn_select = '"join" as id_stn,'
+            group_by = f'GROUP BY 2, 3, 4, 5 {extra_sw}'
+        else:
+            raise ValueError("Unsupported combination of channel and id_stn")
 
         # Interval criteria
         interval_a, interval_b = interval
@@ -438,7 +397,7 @@ def create_and_populate_moyenne_table(
         sum_bias_corr2 = "sum(bias_corr * bias_corr)" if has_bias_corr else "NULL"
 
         # Create table moyenne if not exists
-        cursor.execute("""
+        insert_query = f"""
             CREATE TABLE IF NOT EXISTS moyenne (
                 Nrej INTEGER,
                 Nacc INTEGER,
@@ -460,16 +419,17 @@ def create_and_populate_moyenne_table(
                 sumStat2 FLOAT,
                 n INTEGER,
                 flag INTEGER
-            );
-        """)
-
+                {extra_sw}
+            );"""
+        cursor.execute(insert_query)
         # Compose query
         insert_query = f"""
         INSERT INTO moyenne (
             DATE, lat, lon, boite, varno, vcoord, 
             sumx, sumy, sumz, sumStat, 
             sumx2, sumy2, sumz2, sumStat2, 
-            n, Nrej, Nacc, Nprofile, id_stn, flag
+            n, Nrej, Nacc, Nprofile, id_stn, flag  {extra_sw}
+
         )
         SELECT
             {date_str},
@@ -492,6 +452,7 @@ def create_and_populate_moyenne_table(
             COUNT(DISTINCT id_obs),
             {id_stn_select}
             flag
+            {extra_sw}
         FROM
             db.header
         NATURAL JOIN
@@ -510,7 +471,7 @@ def create_and_populate_moyenne_table(
 
         # Now, merge in-memory results into the output file
         try:
-            combine(new_db_filename, in_memory_db)
+            combine(new_db_filename, in_memory_db, extra_sw )
         except sqlite3.Error as error:
             print(f"Error while creating a single sqlite file: {os.path.basename(in_memory_db)} -- {error}")
 from datetime import datetime, timedelta
@@ -598,81 +559,34 @@ def create_data_list(
         current_date += delta
 
     return data_list   
-def remove_create_data_list(datestart1, 
-                     dateend1,
-                     families,
-                     pathins, 
-                     names, 
-                     pathwork,
-                     boxsizex, 
-                     boxsizey, 
-                     fonction, 
-                     flag_criteria, 
-                     regions, 
-                     varnos,
-                     intervales):
-       data_list = []
-
-       # Convert datestart and dateend to datetime objects
-       datestart = datetime.strptime(datestart1, '%Y%m%d%H')
-       dateend = datetime.strptime(dateend1, '%Y%m%d%H')
-
-       # Initialize the current_date to datestart
-       current_date = datestart
-
-       # Define a timedelta of 6 hours
-       delta = timedelta(hours=6)
-   
-       # Iterate through the date range in 6-hour intervals
-       while current_date <= dateend:
-               for family in families:
-                for region in regions:
-                 for name, pathin in zip(names,pathins):
-                  for interval in intervales:
-               #   for pathin in pathins:
-                    interval_a=interval[0]
-                    interval_b=interval[1]
-                    if  interval_a==None and  interval_b==None:
-                       criteria_interval = ''
-                       layers='layer_all'
-                    else:
-                       criteria_interval = f' and  {interval_a*100} <= vcoord <= {interval_b*100}'
-                       layers=f'layer_from{interval_a}MPato{interval_b}MPa'
-                    # Format the current date as a string
-                    formatted_date = current_date.strftime('%Y%m%d%H')
-   
-                    # Build the file name using the date and family
-                    filename = f'{formatted_date}_{family}'
-                    # Create a new dictionary and append it to the list
-                    data_dict = {
-                      'family': family,
-                      'filein': f'{pathin}/{filename}',
-                      'db_new': f'{pathwork}/{family}/scatter_{layers}_{name}_{region}_{datestart1}_{dateend1}_bx{boxsizex}_by{boxsizey}_{flag_criteria}_{family}.db',
-                      'region': region,
-                      'flag_criteria': flag_criteria,
-                      'fonction': fonction,
-                      'boxsizex': boxsizex,
-                      'boxsizey': boxsizey,
-                      'varnos':varnos,
-                      'interval':interval
-                    }
-                    data_list.append(data_dict)
-   
-                    # Update the current_date in the loop by adding 6 hours
-               current_date += delta
-   
-       return data_list
    
 import sqlite3
 import numpy as np
 
-def get_id_stns(cursor, id_stn):
+def get_id_stns(cursor, id_stn, WIND_COMP_METHOD):
     """Obtiene la lista de id_stn desde la base de datos."""
-    if id_stn == 'all':
+  #  print (id_stn, WIND_COMP_METHOD)
+    if id_stn == 'all' and WIND_COMP_METHOD:
+        #print ('a')
+        query = "SELECT DISTINCT id_stn, WIND_COMP_METHOD FROM moyenne;"
+        cursor.execute(query)
+        return np.array(cursor.fetchall())
+
+    if id_stn == 'all' and WIND_COMP_METHOD != True:
+       # print ('b')
         query = "SELECT DISTINCT id_stn FROM moyenne;"
         cursor.execute(query)
-        return np.array([item[0] for item in cursor.fetchall()])
-    return ['join']
+        rows = cursor.fetchall()
+        return np.array([[row[0], None] for row in rows])
+    if id_stn == 'join' and WIND_COMP_METHOD == True:
+      #  print ('c')
+        query = "SELECT DISTINCT WIND_COMP_METHOD FROM moyenne;"
+        cursor.execute(query)
+        rows = cursor.fetchall()
+        return np.array([['join',row[0]] for row in rows])
+
+
+    return [('join', x) for x in [None, None]]
 
 def fetch_vcoords(cursor, criter):
     """Obtiene las coordenadas verticales y números de variable según el criterio dado."""
@@ -686,75 +600,6 @@ def fetch_channels_varno(cursor):
     cursor.execute(query)
     return [item[0] for item in cursor.fetchall()]
 
-def remove_create_data_list_plot(datestart1, dateend1, families, namein, pathwork, boxsizex, boxsizey,
-                          fonctions, flag_criteria, regions, id_stn, channel, projs,intervales):
-    data_list_plot = []
-    for interval in intervales:
-      for family in families:
-       for region in regions: 
-        for fonction in fonctions:
-         for proj in projs:
-          interval_a = interval[0]
-          interval_b = interval[1]
-          
-          if  interval_a==None and  interval_b==None:
-            criteria_interval = ''
-            layers='layer_all'
-          else:
-            criteria_interval = f' and  {interval_a*100} <= vcoord <= {interval_b*100}'
-            layers=f'layer_from{interval_a}MPato{interval_b}MPa'
-          
-          fileset = [f'{pathwork}/{family}/scatter_{layers}_{namein[0]}_{region}_{datestart1}_{dateend1}_bx{boxsizex}_by{boxsizey}_{flag_criteria}_{family}.db']
-          nameset = [namein[0]]
-      
-          if len(namein) > 1:
-              fileb = f'{pathwork}/{family}/scatter_{layers}_{namein[1]}_{region}_{datestart1}_{dateend1}_bx{boxsizex}_by{boxsizey}_{flag_criteria}_{family}.db'
-              fileset.append(fileb)
-              nameset.append(namein[1])
-          with sqlite3.connect(fileset[0]) as conn:
-              cursor = conn.cursor()
-              id_stns = get_id_stns(cursor, id_stn)
-      
-              for idstn in id_stns:
-                  criter = '' if id_stn == 'join' else f'WHERE id_stn = "{idstn}"'
-                  
-                  if channel == 'all':
-      
-                      vcoords = fetch_vcoords(cursor, criter)
-                      for vcoord, varno in vcoords:
-                          data_list_plot.append({
-                              'id_stn': idstn,
-                              'vcoord': vcoord,
-                              'files_in': fileset,
-                              'varno': varno,
-                              'region': region,
-                              'fonction':fonction,
-                              'proj':proj,
-                              'interval':interval,
-                              'family':family
-
-  
-                          })
-                  else:
-                      channels_varno = fetch_channels_varno(cursor)
-                      #print (channels_varno)
-                      for  varno in channels_varno:
-                     # if channels_varno:
-                          data_list_plot.append({
-                              'id_stn': idstn,
-                              'vcoord': 'join',
-                              'files_in': fileset,
-                              'varno': varno,
-                              'region': region,
-                              'fonction':fonction,
-                              'proj':proj ,
-                              'interval':interval,
-                              'family':family
-
-
-  
-                          })
-    return data_list_plot
 def create_data_list_plot(
     date_start: str,
     date_end: str,
@@ -821,9 +666,13 @@ def create_data_list_plot(
                         try:
                             with sqlite3.connect(fileset[0]) as conn:
                                 cursor = conn.cursor()
-                                id_stns = get_id_stns(cursor, id_stn)    # Returns List[str]
-
-                                for idstn in id_stns:
+                                cursor.execute("PRAGMA table_info('moyenne')")
+                                columns = [col[1] for col in cursor.fetchall()]
+                                WIND_COMP_METHOD = 'WIND_COMP_METHOD' in columns
+                                   
+                                id_stns = get_id_stns(cursor, id_stn, WIND_COMP_METHOD)    # Returns List[str]
+                                for idstn, WIND_COMP_METHOD  in id_stns:
+                                  #  print (idstn, WIND_COMP_METHOD)
                                     where_clause = '' if id_stn == 'join' else f'WHERE id_stn = "{idstn}"'
                                     if channel == 'all':
                                         vcoords = fetch_vcoords(cursor, where_clause)  # Returns List[Tuple[Any, int]]
@@ -837,7 +686,8 @@ def create_data_list_plot(
                                                 'function': function,
                                                 'proj': proj,
                                                 'interval': interval,
-                                                'family': family
+                                                'family': family,
+                                                'WIND_COMP_METHOD':WIND_COMP_METHOD 
                                             })
                                     else:  # channel == 'join' or other
                                         channels_varnos = fetch_channels_varno(cursor)  # Returns List[int]
@@ -851,7 +701,8 @@ def create_data_list_plot(
                                                 'function': function,
                                                 'proj': proj,
                                                 'interval': interval,
-                                                'family': family
+                                                'family': family,
+                                                'WIND_COMP_METHOD':WIND_COMP_METHOD
                                             })
                         except sqlite3.Error as err:
                             print(f"Warning: Could not open or query {fileset[0]}: {err}")
@@ -1005,7 +856,7 @@ def make_scatter(files_in,
                                             )
 
         t0 = time.time()
-        #n_cpu=1
+       # n_cpu=1
         if n_cpu==1:  
          print (f'in Serie plots = {len(data_list_plot)}')
          for  data_ in data_list_plot:  
@@ -1026,7 +877,8 @@ def make_scatter(files_in,
                                data_['files_in'],
                                names_in, 
                                data_['varno'],
-                               data_['interval'])
+                               data_['interval'],
+                               data_['WIND_COMP_METHOD'])
         else:
          print (f"in Paralle plots = {len(data_list_plot)}")
          time.sleep(4)
@@ -1049,7 +901,8 @@ def make_scatter(files_in,
                                                               data_['vcoord'],
                                                               data_['files_in'],
                                                               names_in, data_['varno'],
-                                                              data_['interval'])for data_ in data_list_plot]
+                                                              data_['interval'],
+                                                              data_['WIND_COMP_METHOD'])for data_ in data_list_plot]
    
            results = dask.compute(*delayed_funcs)
            client.close()
