@@ -105,6 +105,44 @@ def round_first_digit_3(x):
         return f"{mantissa}.e{int(exp)}"
 
 import numpy as np
+
+def wind_type(code_or_codes):
+    """
+    Devuelve la(s) descripción(es) del tipo de viento para:
+    - int (un solo código)
+    - list/tuple/set (varios códigos)
+    - pandas.Series (columna de códigos)
+    """
+    mapping = {
+        1: "Wind derived from cloud motion observed in the infrared channel",
+        2: "Wind derived from cloud motion observed in the visible channel",
+        3: "Wind derived from cloud motion observed in the water vapour channel",
+        4: "Wind derived from motion observed in a combination of spectral channels",
+        5: "Wind derived from motion observed in the water vapour channel in clear air",
+        6: "Wind derived from motion observed in the ozone channel",
+        7: "Wind derived from motion observed in water vapour channel (cloud or clear air not specified)",
+    }
+
+    def map_one(c):
+        try:
+            return mapping.get(int(c), f"Unknown code {c}")
+        except Exception:
+            return f"Unknown code {c}"
+
+    # Colecciones comunes
+    if isinstance(code_or_codes, (list, tuple, set)):
+        return [map_one(c) for c in code_or_codes]
+
+    # pandas.Series
+    try:
+        import pandas as pd
+        if isinstance(code_or_codes, pd.Series):
+            return code_or_codes.map(map_one)
+    except Exception:
+        pass
+
+    # Escalar
+    return map_one(code_or_codes)
 def scatter_plot(
                    mode,
                    region,
@@ -123,7 +161,8 @@ def scatter_plot(
                    filesin,
                    namesin,
                    varno,
-                   intervales):
+                   intervales,
+                   condition_sw = None):
 
        selected_flags = pikobs.flag_criteria(flag_criteria)
 
@@ -175,8 +214,16 @@ def scatter_plot(
              info_name = f"{namesin[0]} VS {namesin[1]}"
        else:
              create_table='AVG'
-             info_name = f"namesin[0]"
-        
+             info_name = f"namesin[0]" 
+       criteria_sw = ''
+       name_sw=''
+       layer_sw=''
+      # print ( 'DDDDDDD',condition_sw , condition_sw != None, criteria_sw )
+       if condition_sw != None:
+         criteria_sw = f' and WIND_COMP_METHOD={condition_sw}'
+         name_sw=f'_OBS_SWMT_{condition_sw}_'
+         layer_sw=wind_type(condition_sw)
+    #   print (   {criteria_sw})
        query = f"""CREATE TEMPORARY TABLE {create_table} AS
                    SELECT boite, 
                           lat,
@@ -189,9 +236,8 @@ def scatter_plot(
                           SUM(n) N
                    FROM db1.moyenne
                    where varno={varno}
-
                    {crite} 
-
+                   {criteria_sw}
                    GROUP BY boite, lat, lon, varno having sum(n)>3;"""
        cursor.execute(query)
 
@@ -209,9 +255,9 @@ def scatter_plot(
                        where  varno={varno}
 
                        {crite} 
-                     
-                       GROUP BY boite, lat, lon, varno having sum(n)>3;"""  
+                       {criteria_sw}
 
+                       GROUP BY boite, lat, lon, varno having sum(n)>3;"""  
            cursor.execute(query)
 
            query = f"""Create temporary table AVG as 
@@ -239,7 +285,7 @@ def scatter_plot(
            cursor.execute(query)
        
        query = f"""
-        SELECT lat, lon, avg, std, N
+        SELECT lat, lon, avg, std, BCORR, N 
         FROM AVG;
        """
        if len(filesin) > 1:
@@ -256,9 +302,11 @@ def scatter_plot(
        import numpy as np
        lat = np.array([row[0] for row in results])
        lon = np.array([row[1] for row in results])
-       Bomp = np.array([row[2] for row in results])
-       Somp = np.array([row[3] for row in results])
-       nombre = np.array([row[4] for row in results])
+       avg = np.array([row[2] for row in results])
+       std = np.array([row[3] for row in results])
+       bcorr = np.array([row[4] for row in results])
+       nombre = np.array([row[5] for row in results])
+       
        if len(filesin) > 1:
 
           N1 = np.array([row[5] for row in results])
@@ -292,12 +340,12 @@ def scatter_plot(
        dens = nombre/NPSURFLL(lat-deltay,lat+deltay,lon-deltax,lon + deltax)
 
 
-       index_none=np.where(Somp ==None)
+       index_none=np.where(avg ==None)
        lat = np.delete(lat, index_none) 
-       lon = np.delete(lon, index_none)
-       Bomp = np.delete(Bomp, index_none)
-       Somp = np.delete(Somp, index_none)
-
+       lon = np.delete(lon, index_none)   
+       avg = np.delete(avg, index_none)
+       std = np.delete(std, index_none)
+       bcorr =  np.delete(bcorr, index_none)
        nombre = np.delete(nombre, index_none)
 
        query = f"""select  
@@ -330,8 +378,7 @@ def scatter_plot(
        from matplotlib.ticker import FuncFormatter
        from mpl_toolkits.axes_grid1 import make_axes_locatable
        import cartopy
-       
-       if Sigma is not None:
+       if Sigma[0] is not None:
            Sigma = np.round(Sigma, 3)
        
            vartyp = fonction
@@ -340,22 +387,22 @@ def scatter_plot(
            variable_name, units, vcoord_type = pikobs.type_varno(varno)
        
            if vcoord == 'join':
-               Nomvar = f"{variable_name} {units} \n id_stn:{id_stn} vcoord/channel:{(vcoord)} {layers}"
+               Nomvar = f"{variable_name} {units} \n id_stn:{id_stn} vcoord/channel:{(vcoord)} {layers} \n {layer_sw}"
            else:
                Nomvar = f"{variable_name} {units} \n id_stn:{id_stn} vcoord/channel:{int(vcoord)} {layers} "
        
            mode = 'MOYENNE'
-           OMP = Somp if mode == 'SIGMA' else Bomp
-           OMP = np.nan_to_num(OMP, nan=np.nan)
+         #  OMP = Somp if mode == 'SIGMA' else Bomp
+       #    OMP = np.nan_to_num(OMP, nan=np.nan)
        
            plt.close('all')
            fig = plt.figure(figsize=(10, 10))
            plt.rcParams['axes.linewidth'] = 1
            fontsize = 17
        
-           OMPm = [value for value in OMP if isinstance(value, float)]
-           vmin = np.nanmin(OMPm)
-           vmax = np.nanmax(OMPm)
+        #   OMPm = [value for value in OMP if isinstance(value, float)]
+       #    vmin = np.nanmin(OMPm)
+      #     vmax = np.nanmax(OMPm)
            Ninterv = 10
        
            if vartyp == 'dens' or vartyp == 'dens%' :
@@ -368,8 +415,6 @@ def scatter_plot(
                   vmin = -100
                   vmax =  100
                else:
-                  
-             
                      vmin = np.min(OMP)
                      vmax = np.max(OMP)
                cmap_base = cm.get_cmap('PuRd', lut=Ninterv)
@@ -381,7 +426,6 @@ def scatter_plot(
                OMP = nombre / NDAYS
                if len(filesin) > 1:
                    N1=N1 / NDAYS
-#
                    N2=N2/ NDAYS
 
                    OMP =(N2-N1)*100/N1
@@ -406,6 +450,7 @@ def scatter_plot(
                    vmax= 100
 
                else:
+                  OMP = avg
                   vmin = np.min(OMP)
                   vmax = np.max(OMP)
                   cmap_base = cm.get_cmap('RdYlBu_r', lut=Ninterv)  
@@ -416,58 +461,93 @@ def scatter_plot(
                    vmax= 100
 
                else:
+                  OMP = bcorr
                   vmin = np.min(OMP)
                   vmax = np.max(OMP)
                   cmap_base = cm.get_cmap('RdYlBu_r', lut=Ninterv)
 
-           elif vartyp in ['omp', 'oma', 'bcorr', 'stdomp']:
-               vmin = np.min(OMP)
-               vmax = np.max(OMP)
-
-               if len(filesin) > 1:
-                   
-                   OMP =(abs(avg2)-abs(avg1))*100/abs(avg1)
-                   vmin = -100
-                   vmax = 100
-
-
+           elif vartyp in ['omp', 'oma', 'bcorr', 'stdomp', 'stdoma']:
                N = 10
-               if vartyp == 'stdomp':
-                   mode = 'SIGMA'
-                   typer = 'STD'
-               cmap_base = cm.get_cmap('seismic', lut=N)
-               if mode == 'MOYENNE':
-                   typer = 'AVG'
-               if mode == 'SIGMA':
-                   cmap_base = cm.get_cmap('RdYlBu_r', lut=Ninterv)
+               if len(filesin) > 1:
+                   if vartyp  in ['omp', 'oma'] : 
+                       OMP =(abs(avg2)-abs(avg1))*100/abs(avg1)
+                       vmin = -100
+                       vmax = 100
+                   if vartyp in  ['stdomp', 'stdoma'] :
+                    
+                       OMP =(abs(std2)-abs(std1))*100/abs(std1) 
+                   
+               else:
+                   if vartyp  in ['omp', 'oma'] : 
+                       OMP = avg
+
+                       vmax = np.max(np.abs(OMP))
+                       vmin = -vmax
+                       
+
+                       cmap_base = cm.get_cmap('seismic', lut=N)
+
+
+                   if vartyp == 'stdomp':
+                       OMP = std
+                       mode = 'SIGMA'
+                       typer = 'STD'
+                       vmin = np.min(OMP)
+                       vmax = np.max(OMP)
+                  #     cmap_base = cm.get_cmap('RdYlBu_r', lut=Ninterv)
+                       cmap_base = cm.get_cmap('PuRd', lut=Ninterv)
+
+               #cmap_base = cm.get_cmap('seismic', lut=N)
+               #if mode == 'MOYENNE':
+                #   typer = 'AVG'
+               #if mode == 'SIGMA':
+                 #  cmap_base = cm.get_cmap('RdYlBu_r', lut=Ninterv)
            else:
                cmap_base = cm.get_cmap('RdYlBu_r', lut=Ninterv)
        
            if len(filesin) > 1:
-              N = 21  
-              boundaries = np.linspace(vmin, vmax, N)
-              y = (boundaries[:-1] + boundaries[1:]) / 2
-              neg_colors = [cm.RdYlBu_r(x) for x in np.linspace(0.08, 0.40, N//2)]
-              blanco = (1, 1, 1, 1) 
-              blanco = (1.0, 1.0, 0.6, 1.0) 
-              pos_colors = [cm.RdYlBu_r(x) for x in np.linspace(0.60, 0.92, N//2)]
-              over_color = '#e600c7'   
-              under_color = '#0235ad' 
-              colores =  neg_colors + [blanco] + pos_colors  
+             extend_color = 'both'
+             vmin=-100
+             vmax=100
+             boundaries = np.array(
+                 list(range(vmin, -5, 20)) +   # valores negativos hasta -2
+                 [-5, 5] +                      # rango blanco [-2,2]
+                 list(range(20, vmax+10, 20))   # valores positivos desde 10
+             )          
+             y = boundaries
+
+
+             # Colores negativos
+             neg_colors = [cm.RdYlBu_r(x) for x in np.linspace(0.08, 0.40, len(boundaries[boundaries < -2]))]
+             
+             # Color blanco para [-2,2]
+             blanco = (1.0, 1.0, 1.0, 1.0)
+             
+             # Colores positivos
+             pos_colors = [cm.RdYlBu_r(x) for x in np.linspace(0.60, 0.92, len(boundaries[boundaries > 2]))]
+             
+             # Concatenar colores
+             colores = neg_colors + [blanco] + pos_colors
+             
+             # Crear cmap y norm
+             cmap = colors.ListedColormap(colores)
+             cmap.set_under('#0235ad')
+             cmap.set_over('#b30000')
+             norm = colors.BoundaryNorm(boundaries, cmap.N)
+             
+             # Ticks para el colorbar
+             ticks = list(boundaries[boundaries < -5]) + [-5, 5] + list(boundaries[boundaries > 5])
 
-              cmap = colors.ListedColormap(colores)
-              cmap.set_under('#0235ad')
-              cmap.set_over('#e600c7')
-              norm = colors.BoundaryNorm(boundaries, N)
-              extend_color = 'both'
-              ticks = boundaries
            else:
+
                boundaries = np.linspace(vmin, vmax, Ninterv + 1)
                y = boundaries[:-1] + (boundaries[1] - boundaries[0]) / 2
                cmap = cmap_base
                norm = cm.colors.Normalize(vmin=vmin, vmax=vmax)
                extend_color = None
                ticks = boundaries
+               #Colors = [m.to_rgba(x) for x in y]
+
            m = cm.ScalarMappable(norm=norm, cmap=cmap)
            Colors = [m.to_rgba(x) for x in y]
            inds = np.digitize(OMP, boundaries) - 1
@@ -476,7 +556,8 @@ def scatter_plot(
            nombres = 0
            left, bottom = 0.90, 0.15
            ax, fig, LATPOS, PROJ, pc = pikobs.type_projection(proj)
-           ONMAP = 0
+           ONMAP = 0 
+           ONMAP2 = 0
            POINTS = 'OFF'
            patch_list = []
 
@@ -489,7 +570,9 @@ def scatter_plot(
                mask = (xx >= -0.01) & (xx <= 1.01) & (yy >= -0.01) & (yy <= 1.01)
                if mask:
                    ONMAP += nombre[i]
-                 
+                   if len(filesin) > 1:
+ 
+                       ONMAP2 +=N1[i]
                    if POINTS == 'ON':
                        plt.text(point[0], point[1], int(np.floor(nombre[i])), color="k", fontsize=17, zorder=5, ha='center', va='center', weight='bold')
                    else: 
@@ -500,10 +583,10 @@ def scatter_plot(
                        
                        valor = OMP[i]
 
-                       if valor < vmin:
-                           col = '#0235ad'        
-                       elif valor > vmax:
-                           col = '#e600c7'         
+                       if valor < -100:
+                           col =  '#0235ad' #67001f' #0235ad'   #67001f #053061     
+                       elif valor > 100:
+                           col = '#b30000' #e600c7' #b2182b' #e600c7'         
                        else:
                            ind = np.digitize(valor, boundaries) - 1
                            col = Colors[inds[i]]     
@@ -514,7 +597,9 @@ def scatter_plot(
                        
                        poly = plt.Polygon(points4, fc=col, zorder=4, ec='k', lw=0.2, alpha=1.0)
                        ax.add_patch(poly)
-           for boite, latn, lonn in resultados:
+           if len(filesin) > 1:
+            
+            for boite, latn, lonn in resultados:
              points4 = projectPpoly(PROJ, latn, lonn, deltax, deltay, pc)
 
              poly = plt.Polygon(points4, fc=(0,0,0,1), zorder=4, ec='k', lw=0.8, alpha=1.0)
@@ -564,7 +649,13 @@ def scatter_plot(
                 cb2.ax.set_ylabel(f'Percentage of improvement in {variable_name}  \n relative to {namesin[0]} [%]', fontsize=15, rotation=90, labelpad=20)
            if len(filesin) > 1 and   vartyp == 'bcorr' :
                 cb2.ax.set_ylabel(f'Percentage increase in bcorr \n relative to {namesin[0]} [%]', fontsize=18, rotation=90, labelpad=20)
+           if len(filesin) > 1 and   vartyp == 'stdomp' :
+                cb2.ax.set_ylabel(f'Percentage increase in stdomp \n relative to {namesin[0]} [%]', fontsize=18, rotation=90, labelpad=20)
+           if len(filesin) > 1 and   vartyp == 'stdoma' :
+                cb2.ax.set_ylabel(f'Percentage increase in stdoma \n relative to {namesin[0]} [%]', fontsize=18, rotation=90, labelpad=20)
            
+
+
            if len(filesin) == 1 and   vartyp == 'dens':
                 cb2.ax.set_ylabel(f'sum(dens [nobs/km²])/days', fontsize=18, rotation=90, labelpad=20)
            if len(filesin) == 1 and  vartyp == 'nobs':
@@ -577,7 +668,11 @@ def scatter_plot(
                 cb2.ax.set_ylabel(f'{variable_name}  {units}', fontsize=18, rotation=90, labelpad=20)
            if len(filesin) == 1 and   vartyp == 'bcorr' :
                 cb2.ax.set_ylabel(f' BIAS Correction {units} ', fontsize=18, rotation=90, labelpad=20)
-       
+           if len(filesin) == 1 and   vartyp == 'stdomp' :
+                cb2.ax.set_ylabel(f' STDOMP Correction {units} ', fontsize=18, rotation=90, labelpad=20)
+           if len(filesin) == 1 and   vartyp == 'stdoma' :
+                cb2.ax.set_ylabel(f' STDOMA Correction {units} ', fontsize=18, rotation=90, labelpad=20)
+
            if len(filesin) < 1:
                fontsize = 12
                ax.text(0.00, 1.05, namesin[0], fontsize=fontsize, color='b', transform=ax.transAxes) 
@@ -593,7 +688,6 @@ def scatter_plot(
                    text2_len = len(namesin[1])
                    text2_x = text1_x + text1_len
                    text3_x = text1_x + 0.2
-
                    ax.text(text1_x, y_coord, 'Diff ' + namesin[0] , fontsize=fontsize, color='blue', transform=ax.transAxes)
                    ax.text(text3_x, y_coord, "-" + namesin[1] , fontsize=fontsize, color='red', transform=ax.transAxes)
                else:
@@ -601,29 +695,29 @@ def scatter_plot(
        
            ax.text(0.00 + 20, 1.05, vartyp, fontsize=fontsize, color='k', transform=ax.transAxes)
            ax.text(0.00, 1.02, PERIODE, fontsize=fontsize, color='#3366FF', transform=ax.transAxes)
-           ax.text(0.45, 1.05, Nomvar, fontsize=fontsize, color='k', transform=ax.transAxes, fontweight='bold')
+           ax.text(0.35, 1.05, Nomvar, fontsize=fontsize, color='k', transform=ax.transAxes, fontweight='bold')
        
            ax.text(0.5, -0.08, 'Longitude', transform=ax.transAxes, ha='center', va='top', fontsize=14)
            ax.text(-0.06, 0.5, 'Latitude', transform=ax.transAxes, ha='center', va='bottom', rotation='vertical', fontsize=14)
        
            props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
            if vartyp in [f'dens', 'nobs']:
-               textstr = f'Nobs=%.2i (in %.2f days)' % (ONMAP, NDAYS)
+               textstr = f'Nobs=%.2i (for %.2f days)' % (ONMAP, NDAYS)
            else:
-               textstr = '$\\bar{\\mu}=%.3f$ $\\bar{\\sigma}=%.3f$ \nNobs=%.2i (in %.2f days)' % (Mu, Sigma, ONMAP, NDAYS)
+               textstr = '$\\bar{\\mu}=%.3f$ $\\bar{\\sigma}=%.3f$ \nNobs=%.2i (for %.2f days)' % (Mu, Sigma, ONMAP, NDAYS)
        
            if len(filesin) < 2:
-               ax.text(0.88, 1.15, textstr, transform=ax.transAxes, fontsize=fontsize, verticalalignment='top', bbox=props)
+               ax.text(0.85, 1.17, textstr, transform=ax.transAxes, fontsize=fontsize, verticalalignment='top', bbox=props)
            else:
-               textstr = f'Nobs=%.2i (in %.2f days)' % (ONMAP, NDAYS)
+               textstr = f'Diff Nobs={ONMAP:02d} \nDiff%={((ONMAP/(ONMAP2*NDAYS))*100):.2f}% for {NDAYS:.2f} days'
                ax.text(0.88, 1.15, textstr, transform=ax.transAxes, fontsize=fontsize, verticalalignment='top', bbox=props)
         
            plt.grid(True)
            plt.rcParams['axes.linewidth'] = 2
        
            if vcoord == 'join':
-               plt.savefig(f'{pathwork}/{family}/{fonction}_{proj}_{layers}_id_stn_{id_stn}_{region}_vcoord_{int(vcoord)}_varno{varno}.png', dpi=600, format='png')
+               plt.savefig(f'{pathwork}/{family}/{fonction}_{proj}_{layers}_id_stn_{id_stn}_{region}_vcoord_{vcoord}_varno{varno}{name_sw}.png', dpi=600, format='png')
            else:
-               plt.savefig(f'{pathwork}/{family}/{fonction}_{proj}_{layers}_id_stn_{id_stn}_{region}_vcoord_{int(vcoord)}_varno{varno}.png', dpi=600, format='png')
+               plt.savefig(f'{pathwork}/{family}/{fonction}_{proj}_{layers}_id_stn_{id_stn}_{region}_vcoord_{int(vcoord)}_varno{varno}{name_sw}.png', dpi=600, format='png')
            plt.close(fig) 
        
