@@ -1139,7 +1139,6 @@ def make_scatter(files_in,
                                                  data_['interval'])for data_ in data_list]
               # print ('close0')
                results = dask.compute(*delayed_funcs)
-               print ('close1')
                client.close()
 
         tn= time.time()
@@ -1185,7 +1184,7 @@ def make_scatter(files_in,
                                data_['varno'],
                                data_['interval'])
         else:
-         print (f"in Paralle plots = {len(data_list_plot)}")
+         print (f"in Paralle plots")
          time.sleep(4)
          with dask.distributed.Client(processes=True, threads_per_worker=1, 
                                           n_workers=n_cpu, 
@@ -1252,10 +1251,10 @@ def arg_call():
     parser.add_argument('--region', nargs="+", default='undefined', type=str, help="Region")
     parser.add_argument('--family', nargs="+", default='undefined', type=str, help="Family")
     parser.add_argument('--flags_criteria', default='undefined', type=str, help="Flags criteria")
-    parser.add_argument('--fonction', nargs="+", default='undefined', type=str, help="Function") 
+    parser.add_argument('--fonction', nargs="+", default='omp', type=str, help="Function") 
     parser.add_argument('--varnos', nargs="+", default='undefined', type=str, help="Function")
-    parser.add_argument('--boxsizex', default='undefined', type=int, help="Box size in X direction")
-    parser.add_argument('--boxsizey', default='undefined', type=int, help="Box size in Y direction")
+    parser.add_argument('--boxsizex', default=2, type=int, help="Box size in X direction")
+    parser.add_argument('--boxsizey', default=2, type=int, help="Box size in Y direction")
     parser.add_argument('--projection', nargs="+", default='cyl', type=str, help="Projection type (cyl, OrthoN, OrthoS, robinson, Europe, Canada, AmeriqueNord, Npolar, Spolar, reg)")
     parser.add_argument('--mode', default='SIGMA', type=str, help="Mode")
     parser.add_argument('--Points', default='OFF', type=str, help="Points")
